/* GCC */
//read some comments below if you can't solve the problem
// here advise.pdf means 31.pdf file from kievoi
#include <iostream>
#include <fstream>
#include <vector>
#include <cmath>
#include <algorithm>

using namespace std;

#define X first
#define Y second

typedef pair<int, int> DOT;
vector<DOT> dots;
vector<vector<int> > dotsBetween, bestForLower, bestForUpper;
vector< vector<vector<int> > > dpForLower, dpForUpper;
int n, ansAmount=0;

int scalarProduct       (DOT &a, DOT &b, DOT &c){
    return (b.X-a.X)*(c.Y-a.Y) - (c.X-a.X)*(b.Y-a.Y);
}
bool clockWise          (DOT &a, DOT &b, DOT &c){
    return scalarProduct(a,b,c)<0;
}
bool counterClockWise   (DOT &a, DOT &b, DOT &c){
    return scalarProduct(a,b,c)>0;
}
bool sameLine           (DOT &a, DOT &b, DOT &c){
    return scalarProduct(a,b,c)==0;
}

void readData(){
    freopen("fortress.in", "r", stdin);

    scanf("%d/n", &n);
    dots.reserve(n);
    int z, k;
    for(int i=0; i<n; i++){
        scanf("%d %d\n", &z, &k);
        dots.push_back( make_pair(z, k) );
    }

    sort(dots.begin(), dots.end());
    bestForLower=bestForUpper=dotsBetween=vector< vector<int> >(n, vector<int>(n, 0));
    dpForLower=dpForUpper=vector< vector<vector<int> > >(n, vector<vector<int> > (n, vector<int>(n, 0)) );

    for(int i=0; i<n-1; i++){
        for(int z=i+1; z<n; z++){
            dotsBetween[i][z]=2;
            for(int j=i+1; j<z; j++){
                if(sameLine(dots[i], dots[j], dots[z]))
                    dotsBetween[i][z]++;
            }
        }
    }

    fclose(stdin);
}

// Here we write ONE function to calculate both biggest lines : upper and lower.
// check() is either clockWise(for upper) or counterClockWise (for lower).
// dp is either dpForLower or dpForUpper (function F from advise.pdf).
// The same stuff with best ( function M from advise.pdf ).
// you are welcome)
void findBiggestLine(bool (*check)(DOT&, DOT&, DOT&),
                     vector< vector< vector<int> > > &dp,
                     vector< vector<int> > &best,
                     int first, int last)
{
    int currBest=0;
    for(int i=first+1; i<last; i++){
        if( !check(dots[first], dots[i], dots[last])
         || dotsBetween[i][last]>2) continue;
        //calc G from advise.pdf
        int currG=0;
        for(int k=first+1; k<i; k++){
            if( check(dots[k], dots[i], dots[last])
             || sameLine(dots[k], dots[i], dots[last]))
                currG=max(currG, dp[first][k][i]);
        }
        dp[first][i][last]=max(dotsBetween[first][i], currG)+1;
        currBest=max(currBest, dp[first][i][last]);

    }
    best[first][last]=currBest;
}

void solve(){
    for(int i=0; i<n-1; i++){
        for(int z=i+1; z<n; z++){

            // here we calculate best figures from i to z (function M from advise.pdf
            findBiggestLine(       clockWise, dpForUpper, bestForUpper, i, z);
            findBiggestLine(counterClockWise, dpForLower, bestForLower, i, z);

            if(bestForLower[i][z])
                ansAmount=max(ansAmount, bestForLower[i][z]+dotsBetween[i][z]-2);
            if(bestForUpper[i][z])
                ansAmount=max(ansAmount, bestForUpper[i][z]+dotsBetween[i][z]-2);
            ansAmount=max(ansAmount, bestForUpper[i][z]+bestForLower[i][z]-2);
        }
    }
}

void fileOut(){
    freopen("fortress.out", "w", stdout);
    printf("%d\n", ansAmount);
    fclose(stdout);
}

int main(){
    readData();
    solve();
    fileOut();
    return 0;
}
